# Base.pm - The Object Class that provides a Base Object
# Created by James A. Pattie, 2004-12-13.
# Last edited 2004-12-13.

# Copyright (c) 2000-2005 Xperience, Inc. http://www.pcxperience.com/
# All rights reserved.  This program is free software; you can redistribute it
# and/or modify it under the same terms as Perl itself.

package Portal::Objects::Base;
use strict;
use overload '==' => \&objsEqual,
             '!=' => \&objsNotEqual,
             '""' => \&printStr;
use Portal::Base;
use Portal::Data::Variables;
use HTMLObject::Form;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.02';

=head1 NAME

Base - Object used to build a base Object Class.

=head1 SYNOPSIS

 package Portal::Objects::Test;
 use Portal::Objects::Base;
 use strict;
 use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

 require Exporter;

 @ISA = qw(Portal::Objects::Base Exporter AutoLoader);
 @EXPORT = qw();

 $VERSION = '0.01';

 sub new
 {
   my $class = shift;
   my $self = $class->SUPER::new(@_);
   my %args = ( something => 'Hello World!', @_ );

   if ($self->didErrorOccur)
   {
     $self->prefixError();
     return $self;
   }

   # specify whether or not we are using the HTMLObject::Form.
   $self->{usingForms} = 0;

   # specify whether or not we need to still generate the
   # $self->{variablename} entries in populate() to support old
   # code that is still referencing $obj->{variablename} instead
   # of doing $obj->variablename().
   $self->{backwardsCompatible} = 1;

   # instantiate anything unique to this module
   $self->{objName} = "Test";   # specify your objects name here.

   # define your data variables and their types.
   # this must be in the form called main, even if you are not using
   # forms or do not want to display a form on the screen called main,
   # the only way for me to know what entries in the object are data
   # elements and to know what types they are is to define them in
   # the data->{main} hash.  At a minimum, you must have the -type and
   # -Value entries defined, though if you want to use main to display
   # via the HTMLObject::Form module, you will need to define the other
   # attributes like -Type, -Label, etc.
   $self->{data}->{main} =
     { "fname" => { -type => "string", -Value => "" },
       "lname" => { -type => "string", -Value => "" },
       "age" => { -type => "int", -Value => ""},
     };

   # do validation - ONLY if not being derived from!
   # The $self->Portal::Objects::Test::isValid makes sure we access our
   # isValid method and not any classes isValid method that has
   # derived from us.
   if (!$self->Portal::Objects::Test::isValid)
   {
     # the error is set in the isValid() method.
     return $self;
   }

   # do anything else you might need to do.
   return $self;
 }

 sub isValid
 {
   my $self = shift;

   # make sure our Parent class is valid.
   if (!$self->SUPER::isValid())
   {
     $self->prefixError();
     return 0;
   }

   # validate our parameters.
   if ($self->{fname} !~ /^(.+)$/)
   {
     $self->invalid("fname", $self->{fname});
   }

   if ($self->numInvalid() > 0 || $self->numMissing() > 0)
   {
     $self->error($self->genErrorString("all"));
     return 0;
   }
   return 1;
 }

=head1 DESCRIPTION

Base is a base object class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(void)

 Creates a new instance of the Portal::Objects::Base module.
 See Portal::Base(3) for a listing of required arguments.

 Variables defined by this class:

 backwardsCompatible - boolean value that indicates if the
   populate method needs to generate the $self-> entries
   for code that has not yet been updated to reference the
   data by $obj->entry().  Defaults to 0 (false) to force
   the new behaviour.

 objName - Name of the object ("Base", "User", etc.)
 data - hash of data structures used by the HTMLObject::Form.
   Each entry is named after the form.
   The entry named 'main' is special in that it defines the
   elements that this Object stores and will use for
   object comparison and display.
 data->{main} - hash of variables that make up the actual data
   elements of this object.
   The keys are the names of the variables and the value is an
   anonymous hash with at least the following attributes:
     -type - specifies the type of the element for comparison
       purposes.  Valid types are:
         string
         int
         float
         hash
         array
         object
      Currently only the string, int and float types are
      used when checking for object equality and displaying
      the object as text, html or xml.

     -Value - specifies the value associated with this element.
       If specified when defining the 'main' form then this will
       be the default value for this element, thus eliminating the
       need to define default values for %args in the populate()
       method.

   Ex:
   $self->{data}->{main} =
     { "uname" => { -type => "string", -Value => "" },
       "admin" => { -type => "int", -Value => "0" },
       "passwd" => { -type => "string", -Value => "" },
       "colors" => { -type => "array", -Value => [ "blue", "red"] },
     };

 variables - Portal::Data::Variables object.

 formObj - HTMLObject::Form object.

 usingForms - boolean that indicates if the object is using
   the HTMLObject::Form for manipulation and/or displayall.
   Defaults to 1 (true).

 template - hash of templates used by the HTMLObject::Form.
   Each entry is named after the form.
 profile - hash of profile structures used by the HTMLObject::Form.
   Each entry is named after the form.
 javascript - hash of javascript code blocks grouped by form name,
   which are passed into the generate() method.

 Even if your code only is using one form, you still have to name it
 and index the template, data and profile hashes with the form name.
 This allows for easy expansion in the future, without having to
 go back and name forms later, etc.

 defaultForm - the default name of the form to use when printing the
   object, etc.  This is only used if usingForms is 1 (true).
   Defaults to 'main'.

 formMethod - the default method forms are submitted as.
   Valid values are: post, get
   Default value is 'post'.

 formAction - the default action string for your forms.
   Default value is ''.


 You now have to use the populate() method to assign values to the
 parameters and have them be validated.

 new() just instantiates the object and does not do any initial
 assigning to forms, variables, etc.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = (  @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{backwardsCompatible} = 0;
  $self->{usingForms} = 1;
  $self->{defaultForm} = "main";
  $self->{template} = {};
  $self->{data} = {};
  $self->{data}->{main} = {};  # pre-create the 'main' form hash for the user.
  $self->{profile} = {};
  $self->{objName} = "Base";
  $self->{formMethod} = "post";
  $self->{formAction} = "";

  $self->{variables} = Portal::Data::Variables->new(langObj => $self->{langObj});
  if ($self->{variables}->error)
  {
    $self->error($self->{variables}->errorMessage);

    return $self;
  }

  $self->{formObj} = HTMLObject::Form->new();
  if ($self->{formObj}->error)
  {
    $self->error($self->{formObj}->errorMessage);

    return $self;
  }

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if ($self->{usingForms} !~ /^(0|1)$/)
  {
    $self->invalid("usingForms", $self->{usingForms}, "Must be a boolean value (0 or 1).");
  }

  if (length $self->{objName} == 0)
  {
    $self->missing("objName");
  }

  if (! exists $self->{data})
  {
    $self->missing("data");
  }
  elsif (! exists $self->{data}->{main})
  {
    $self->missing("main", "You must define form 'main'!");
  }
  elsif (scalar keys %{$self->{data}->{main}} == 0)
  {
    $self->missing("main", "You must define the variables that make up your object.");
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self);
  if (!ref($self))
  {
    my $i=0;
    my $result = "";
    while (my @info=caller($i++))
    {
      $result .= "$info[2] - $info[3]: $info[4]\n";
    }
    die "$self is not an object\nCalled from:\n$result";
  }
  my $name = $AUTOLOAD;

  # make sure we don't emulate the DESTROY() method.
  return if $name =~ /::DESTROY$/;

  $name =~ s/.*://; # strip fully-qualified portion
  if (exists $self->{$name})
  {
    if (@_)
    {
      return $self->{$name} = shift;
    }
    else
    {
      return $self->{$name};
    }
  }
  elsif (exists $self->{data}->{main}->{$name})
  {
    if (@_)
    {
      return $self->{data}->{main}->{$name}->{-Value} = shift;
    }
    else
    {
      return $self->{data}->{main}->{$name}->{-Value};
    }
  }
  die "Can't access `$name' field in object of class $type";
}

=item bool objsEqual(a,b)

 returns 1 if the 2 objects are the same (content wise),
 0 otherwise.

=cut
sub objsEqual
{
  my $a = shift;
  my $b = shift;

  if (ref($a) ne ref($b))
  {
    die "objsEqual: a = '" . ref($a) . "' NOT EQUAL b = '" . ref($b) . "'";
  }

  # only compare those elements defined in the data->{main} hash.
  # first we walk a then b, to make sure that they both have the same number of elements, etc.

  foreach my $key (keys %{$a->{data}->{main}})
  {
    return 0 if (! exists $b->{data}->{main}->{$key});
    if ($a->{data}->{main}->{$key}->{-type} =~ /^(string)$/)
    {
      return 0 if ($a->{data}->{main}->{$key}->{-Value} ne $b->{data}->{main}->{$key}->{-Value});
    }
    elsif ($a->{data}->{main}->{$key}->{-type} =~ /^(int|float)$/)
    {
      return 0 if ($a->{data}->{main}->{$key}->{-Value} != $b->{data}->{main}->{$key}->{-Value});
    }
  }

  foreach my $key (keys %{$b->{data}->{main}})
  {
    return 0 if (! exists $a->{data}->{main}->{$key});
    if ($b->{data}->{main}->{$key}->{-type} =~ /^(string)$/)
    {
      return 0 if ($a->{data}->{main}->{$key}->{-Value} ne $b->{data}->{main}->{$key}->{-Value});
    }
    elsif ($b->{data}->{main}->{$key}->{-type} =~ /^(int|float)$/)
    {
      return 0 if ($a->{data}->{main}->{$key}->{-Value} != $b->{data}->{main}->{$key}->{-Value});
    }
  }

  return 1;  # assume they are equal if we get this far.
}

=item bool objsNotEqual(a,b)

 returns 0 if the 2 objects are the same (content wise),
 1 otherwise.

=cut
sub objsNotEqual
{
  my $a = shift;
  my $b = shift;

  if (ref($a) ne ref($b))
  {
    die "objsEqual: a = '" . ref($a) . "' NOT EQUAL b = '" . ref($b) . "'";
  }

  # take advantage of the == overload to do the brunt of our work.
  return (!($a == $b));
}

=item bool populate()

 Always populates the data->{main} structure.

 If usingForms = 1,

   Populates the other forms defined in data.

 End If

 returns: true/false indicating isValid() result.

=cut
sub populate
{
  my $self = shift;
  my %args = ( @_ );

  # update those entries in the data->{main} hash.
  # also for backwards compatibility, until the apps can be updated, create the $self-> entry so $obj->{var} works.
  foreach my $name (keys %{$self->{data}->{main}})
  {
    # handle the case where the user is using main as their form and this is a checkbox entry.
    if (exists $self->{data}->{main}->{$name}->{-Type} && $self->{data}->{main}->{$name}->{-Type} eq "checkbox")
    {
      if (exists $args{$name})
      {
        $self->{data}->{main}->{$name}->{checked} = 1;
      }
      else
      {
        $self->{data}->{main}->{$name}->{checked} = 0;
      }
      $self->{$name} = $self->{data}->{main}->{$name}->{-Value} if ($self->{backwardsCompatible});  # use the default value, not what the user specified.
    }
    else
    {
      $self->{data}->{main}->{$name}->{-Value} = $args{$name} if ($self->{backwardsCompatible});
      $self->{$name} = $args{$name};
    }
  }

  if ($self->{usingForms})
  {
    # update the data profiles.
    foreach my $profile (keys %{$self->{data}})
    {
      next if ($profile eq "main");  # skip the main form since we already populated it.
      foreach my $name (keys %{$self->{data}->{$profile}})
      {
        if ($self->{data}->{$profile}->{$name}->{-Type} eq "checkbox")
        {
          if (exists $args{$name})
          {
            $self->{data}->{$profile}->{$name}->{checked} = 1;
          }
          else
          {
            $self->{data}->{$profile}->{$name}->{checked} = 0;
          }
        }
        else
        {
          $self->{data}->{$profile}->{$name}->{-Value} = $args{$name};
        }
      }
    }
  }

  # do validation
  return $self->isValid;
}

=item string printXML(fullXML bool, indent int)

 This routine returns an XML snippet for this object for use in such
 things as exporting data

 fullXML - defaults to 0 (false).  If true, we output the xml header tag.
 indent - defaults to 2.  Specifies how many spaces in our top level tag
          needs to be.

=cut
sub printXML
{
  my $self = shift;
  my $result = "";
  my %args = ( fullXML => 0, indent => 2, @_ );
  my $full = $args{fullXML};
  my $indent = ($full ? 0 : $args{indent});  # start at 0 indentation if we are making a full XML document.
  my $spaceStr = " " x $indent;

  if ($full)
  {
    $result .= "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>";
  }
  $result .= $spaceStr . "<$self->{objName}>\n";
  foreach my $name (keys %{$self->{data}->{main}})
  {
    # skip the non-data parts of the object.
    next if ($self->{data}->{main}->{$name}->{-type} !~ /^(string|int|float)$/);
    $result .= $spaceStr . "  <$name>$self->{data}->{main}->{$name}->{-Value}</$name>\n";
  }
  $result .= $spaceStr . "</$self->{objName}>\n";

  return $result;
}

=item scalar printStr()

 returns the printable version of this object.

=cut
sub printStr
{
  my $self = shift;
  my $out = "Portal::Objects::$self->{objName} : ";

  foreach my $name (keys %{$self->{data}->{main}})
  {
    # skip the non-data parts of the object.
    next if ($self->{data}->{main}->{$name}->{-type} !~ /^(string|int|float)$/);

    $out .= ", " if ($out =~ /=/);  # output a , if we have an = in the string.
    $out .= "$name = '" . $self->{data}->{main}->{$name}->{-Value} . "'";
  }

  $out .= "\n";

  return $out;
}

=item string printHTML(full => bool)

 This routine returns a string of html that represents this object.
 If full => 1 then a complete HTML Document will be returned.

=cut
sub printHTML
{
  my $self = shift;
  my $result = "";
  my %args = ( full => 0, @_ );
  my $full = $args{full};

  $result .= "<p>\n";
  $result .= "  <b>Contents of $self->{objName}</b><br /><br />\n";
  foreach my $name (keys %{$self->{data}->{main}})
  {
    # skip the non-data parts of the object.
    next if ($self->{data}->{main}->{$name}->{-type} !~ /^(string|int|float)$/);

    $result .= "  <b>$name</b>: $self->{data}->{main}->{$name}->{-Value}<br />\n";
  }
  $result .= "</p>\n";
  if ($full)
  {
    my $doc = HTMLObject::Base->new();
    $doc->setTitle($self->{objName});
    $doc->print($result);
    $result = $doc->display(print => 0);
  }
  return $result;
}

=item %form printForm(form => "mainForm", action => "", method => "post", app =>, state =>, command =>)

  The form specified by form will be generated using the
  template, data, profile structures in the object.
  If you do not specify a form, then the defaultForm value will
  be used.

  If you specify app, state, command, then they will be updated
  in the data structure before the form is generated.

  You can specify any other arguments that the generate() method
  accepts and they will be passed through to the generate() method.

=cut
sub printForm
{
  my $self = shift;
  my %args = ( form => $self->{defaultForm}, @_ );
  my $formName = $args{form};
  my %return = ();

  if (!exists $self->{data}->{$formName})
  {
    $self->invalid("form", $formName, "form doesn't exist!");
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return %return;
  }

  foreach my $arg (qw( app state command ))
  {
    if (exists $args{$arg})
    {
      $self->{data}->{$formName}->{$arg}->{-Value} = $args{$arg};
      delete $args{$arg};
    }
  }

  %return = $self->{formObj}->generate(
    data => $self->{data}->{$formName}, template => $self->{template}->{$formName},
    name => $formName, action => $self->{formAction},
    method => $self->{formMethod}, profile => $self->{profile}->{$formName},
    javascript => $self->{javascript}->{$formName}, %args);

  if ($self->{formObj}->error)
  {
    $self->error($self->{formObj}->errorMessage()); return undef;
  }
  return %return;
}

=item boolean validateForm(input => \%input, form =>)

 Defaults to $self->{defaultForm} if form not specified.
 Returns 1 if form is valid, 0 otherwise.

=cut
sub validateForm
{
  my $self = shift;
  my %args = ( input => undef, form => $self->{defaultForm}, @_ );
  my $formName = $args{form};

  if (!exists $self->{data}->{$formName})
  {
    $self->invalid("formName", $formName, "form does not exist!");
  }

  if (! defined $args{input})
  {
    $self->missing("input");
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  $self->{formObj}->validate(input => $args{input}, profile => $self->{profile}->{$formName} );
  if ($self->{formObj}->error)
  {
    $self->error($self->{formObj}->errorMessage);
    return 0;
  }

  return 1;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

James A. Pattie (mailto:james@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Base(3)

=cut
